@@ -158,10 +158,6 @@ impl<B: AsRef<[u8]> + AsMut<[u8]> + Copy> Decode for CopyableBytesDecoder<B> {
         }
     }
 
-    fn is_idle(&self) -> bool {
-        self.offset == 0
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         ByteCount::Finite((self.bytes.as_ref().len() - self.offset) as u64)
     }
@@ -235,10 +231,6 @@ impl<B: AsRef<[u8]> + AsMut<[u8]>> Decode for BytesDecoder<B> {
         }
     }
 
-    fn is_idle(&self) -> bool {
-        self.bytes.is_none()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         ByteCount::Finite(self.exact_requiring_bytes())
     }
@@ -287,10 +279,6 @@ impl Decode for RemainingBytesDecoder {
         }
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.is_empty()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         ByteCount::Infinite
     }
@@ -419,10 +407,6 @@ where
         }
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.is_idle()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         self.0.requiring_bytes()
     }
