@@ -44,10 +44,6 @@ where
         track!(self.inner.decode(buf, eos)).map(|(n, r)| (n, r.map(&self.map)))
     }
 
-    fn is_idle(&self) -> bool {
-        self.inner.is_idle()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         self.inner.requiring_bytes()
     }
@@ -89,10 +85,6 @@ where
             .map_err(|e| (self.map_err)(e).into())
     }
 
-    fn is_idle(&self) -> bool {
-        self.inner.is_idle()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         self.inner.requiring_bytes()
     }
@@ -192,10 +184,6 @@ where
         }
     }
 
-    fn is_idle(&self) -> bool {
-        self.inner0.is_idle() && self.inner1.is_none()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         if let Some(ref d) = self.inner1 {
             d.requiring_bytes()
@@ -429,10 +417,6 @@ impl<D: Decode> Decode for Omittable<D> {
         }
     }
 
-    fn is_idle(&self) -> bool {
-        self.do_omit || self.inner.is_idle()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         if self.do_omit {
             ByteCount::Finite(0)
@@ -517,10 +501,6 @@ where
         Ok((size, None))
     }
 
-    fn is_idle(&self) -> bool {
-        self.items.is_none()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         self.inner.requiring_bytes()
     }
@@ -612,10 +592,6 @@ impl<D: Decode> Decode for Length<D> {
         Ok((size, item))
     }
 
-    fn is_idle(&self) -> bool {
-        self.remaining_bytes == self.expected_bytes
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         ByteCount::Finite(self.remaining_bytes)
     }
@@ -739,10 +715,6 @@ where
         Ok((size, None))
     }
 
-    fn is_idle(&self) -> bool {
-        self.items.is_none()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         if self.remaining_items == 0 {
             ByteCount::Finite(0)
@@ -788,10 +760,6 @@ where
         }
     }
 
-    fn is_idle(&self) -> bool {
-        self.inner.is_idle()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         self.inner.requiring_bytes()
     }
@@ -845,10 +813,6 @@ impl<D: Decode> Decode for SkipRemaining<D> {
         }
     }
 
-    fn is_idle(&self) -> bool {
-        self.item.is_none() && self.inner.is_idle()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         if self.item.is_none() {
             self.inner.requiring_bytes()
@@ -950,10 +914,6 @@ impl<D: Decode> Decode for MaxBytes<D> {
         Ok((size, item))
     }
 
-    fn is_idle(&self) -> bool {
-        self.inner.is_idle()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         self.inner.requiring_bytes()
     }
@@ -1018,10 +978,6 @@ where
         Ok((size, item))
     }
 
-    fn is_idle(&self) -> bool {
-        self.inner.is_idle()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         self.inner.requiring_bytes()
     }
@@ -1263,10 +1219,6 @@ impl<D: Decode> Decode for Slice<D> {
         Ok((size, item))
     }
 
-    fn is_idle(&self) -> bool {
-        self.inner.is_idle()
-    }
-
     fn requiring_bytes(&self) -> ByteCount {
         self.inner.requiring_bytes()
     }
@@ -1412,11 +1364,67 @@ impl<E: Encode> Encode for LastItem<E> {
     }
 }
 
+/// Combinator for ignoring EOS if there is no item being decoded.
+///
+/// This is created by calling `DecodeExt::maybe_eos`.
+#[derive(Debug, Default)]
+pub struct MaybeEos<D> {
+    inner: D,
+    started: bool,
+}
+impl<D> MaybeEos<D> {
+    /// Returns a reference to the inner decoder.
+    pub fn inner_ref(&self) -> &D {
+        &self.inner
+    }
+
+    /// Returns a mutable reference to the inner decoder.
+    pub fn inner_mut(&mut self) -> &mut D {
+        &mut self.inner
+    }
+
+    /// Takes ownership of this instance and returns the inner decoder.
+    pub fn into_inner(self) -> D {
+        self.inner
+    }
+
+    pub(crate) fn new(inner: D) -> Self {
+        MaybeEos {
+            inner,
+            started: false,
+        }
+    }
+}
+impl<D: Decode> Decode for MaybeEos<D> {
+    type Item = D::Item;
+
+    fn decode(&mut self, buf: &[u8], mut eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        if !self.started && buf.is_empty() && eos.is_reached() {
+            eos = Eos::new(false);
+        }
+
+        let (size, item) = track!(self.inner.decode(buf, eos))?;
+        if let Some(item) = item {
+            self.started = false;
+            Ok((size, Some(item)))
+        } else {
+            if size != 0 {
+                self.started = true;
+            }
+            Ok((size, None))
+        }
+    }
+
+    fn requiring_bytes(&self) -> ByteCount {
+        self.inner.requiring_bytes()
+    }
+}
+
 #[cfg(test)]
 mod test {
     use {Decode, DecodeExt, Encode, EncodeExt, Eos, ErrorKind};
     use bytes::{Utf8Decoder, Utf8Encoder};
-    use fixnum::{U8Decoder, U8Encoder};
+    use fixnum::{U16beDecoder, U8Decoder, U8Encoder};
     use io::{IoDecodeExt, IoEncodeExt};
 
     #[test]
@@ -1575,4 +1583,17 @@ mod test {
         let (_, item) = track_try_unwrap!(decoder.decode(b"\x03foo", Eos::new(false)));
         assert_eq!(item, Some("foo".to_owned()));
     }
+
+    #[test]
+    fn maybe_eos_works() {
+        let mut decoder = U16beDecoder::new();
+        assert!(decoder.decode(&[][..], Eos::new(true)).is_err());
+
+        let mut decoder = U16beDecoder::new().maybe_eos();
+        assert!(decoder.decode(&[][..], Eos::new(true)).is_ok());
+
+        let mut decoder = U16beDecoder::new().maybe_eos();
+        assert!(decoder.decode(&[1][..], Eos::new(false)).is_ok());
+        assert!(decoder.decode(&[][..], Eos::new(true)).is_err());
+    }
 }
